#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jul 18 12:02:46 2018
"""

# Author: Chendi Huang <elimpalm@gmail.com>
#         Xinwei Sun <sxwxiaoxiao@pku.edu.cn>
#         Jiechao Xiong <xjctzdyh@126.com>
#         Yuan Yao <yuany@ust.hk>

# References:
#
#    Huang C, Sun X, Xiong J, et al.
#    Split LBI: An Iterative Regularization Path with Structural Sparsity.
#    Advances In Neural Information Processing Systems. 2016: 3369-3377.
#
#    Vaiter S, Peyré G, Dossal C, et al. Robust sparse analysis regularization.
#    IEEE Transactions on information theory, 2013, 59(4): 2001-2016.
#
#    Osher S, Ruan F, Xiong J, et al.
#    Sparse recovery via differential inclusions.
#    Applied and Computational %Harmonic Analysis, 2016, 41(2): 436-469.





import numpy as np
from sklearn.model_selection import StratifiedKFold,KFold
from .base import predict_error_gaussian,predict_error_binomial
from .linear import LBI_Linear
from .logistic import LBI_Logit


def LB(X,y,ind_true=None,auc_compute=False,roc_plot=True,path_plot=True,family='gaussian',kappa=10,
       alpha=None,t0=None,t_ratio=100,t_num=500,t_seq=None,fit_intercept=False,normalize=False,
       copy_X=True,fast_init=False,group_index=None,verbose=True):
     
     """
     Linearized Bregman Iteration for l1 Problem.
     
     Parameters
     ----------
     X: array, the design matrix, with shape (n_samples,n_features)
     y: array, the label, with shape (n_samples,)
     ind_true: array, the true signal set (for auc computation), default None since it's unknown
          it should be set if the auc_compute is true.
     auc_compute: boolean, optional, default False
         whether compute auc to measure the model selection consistency.
     roc_plot: boolean, optional, default True
         whether plot roc curve when compute auc
     path_plot: boolean, optional, default True
         whether plot regularization solution path
     family: string, "gaussian" or "binomial", default "gaussian"
         the distribution of y
     kappa : int, default 10, Damping factor
     alpha : float, the step size which satisfies
         kappa * alpha * || X^{\star}X ||_2 < 2
     t0 : float, the regularization parameter at which
         the non-zeros variables start to come out.
         It can be determined by calculating the 
         MLE estimator
     t_ratio: float, default 100,
          the length of the path: t0 * t_ratio.
     t_num: int, Default 500, 
          the number of saved estimators in the path. 
     t_seq: array, the saved estimators in the path. 
     fit_intercept : boolean, optional, default True
         whether to calculate the intercept during the path. If set
         to False, no intercept will be used in calculations
         (e.g. data is expected to be already centered).
     normalize : boolean, optional, default False
         This parameter is ignored when ``fit_intercept`` is set to False.
         If True, the regressors X will be normalized before regression by
         subtracting the mean and dividing by the l2-norm.
         If you wish to standardize, please use
         :class:`sklearn.preprocessing.StandardScaler` before calling ``fit`` on
         an estimator with ``normalize=False``.
     copy_X : boolean, optional, default True
         If True, X will be copied; else, it may be overwritten.
     fast_init : boolean, optional, default False
         Whether use the MLE estimator of intercept as initialization
     group_index: array, optional, default None
         specify the group of each variable
     verbose: boolean, optional, default True
         whether show the procession of iteration.
         
     Attributes
     ----------
     beta_ : array, shape (n_features, t_num)
        T he sparse estimators in the regularization path.
     intercept_ : array shape (t_num, )
         Independent term in the regularization path.
     z_: array, shape (n_features, t_num)
         The auxiliary estimators (Defined as: rho + beta / kappa)
         in the regularization path.
     alpha: float, step size
     t_seq: array, shape (t_num, )
          The regularization parameters at which we save the estimators
     var_hist: array, the record of selected variable (non-zeros) in the path
     var_order: array, the step of when the non-zeros variables are selected
    
     """
     class_dict = {'gaussian':LBI_Linear,'binomial':LBI_Logit}
     
     
     obj = class_dict.get(family)(kappa,alpha,t0,t_ratio,t_num,t_seq,fit_intercept,
                         normalize,copy_X,fast_init,group_index,verbose)
     obj.fit(X,y)
     
     if auc_compute is True:
          if ind_true is None:
               raise ValueError('Should provide the true signal set if compute AUC!')
          else:
               obj.auc = obj.ROC_AUC(ind_true,roc_plot)
     
     if path_plot is True:
          obj.Solution_path(ind_true)
     
     return obj


             
def cv_LB(X,y,kfolds=5,random_seed=1,ind_true=None,auc_compute=False,roc_plot=True,path_plot=True,
          family='gaussian',kappa=10,alpha=None,t0=None,t_ratio=100,t_num=500,t_seq=None,
          fit_intercept=False,normalize=False,copy_X=True,fast_init=False,
          group_index=None,verbose=True):
    """
    Cross validation
    
    Parameters (in addition to the ones in LB)
    ----------
    kfolds: the number of folds in cross-valdation
    random_seed: the random seed for partition of the dataset
    
    Attributes (in addtion to the ones in LB):
    ----------
    predict_error_cv: the prediction error by cross-validation
    ind_cv: the regularization parameter which returns the minimum prediction error
    """
    
    ## Kfolds
    if isinstance(kfolds,int) is False or kfolds < 2:
         raise ValueError('The kfolds must be int and greater than 2!')
    if kfolds > X.shape[0]:
         raise ValueError('The kfolds should be less or equal than the number of samples!')
        
    ## Random Seed for partition
    np.random.seed(random_seed)
    
    ## Partition (X,y) into kfolds
    partition_dict = {'gaussian':KFold,'binomial':StratifiedKFold}
    skf = partition_dict.get(family)(n_splits=kfolds)
    skf.get_n_splits(X, y)
    
    ## Calculate prediction error on train dataset
    class_dict = {'gaussian':LBI_Linear,'binomial':LBI_Logit}
    obj = class_dict.get(family)(kappa,alpha,t0,t_ratio,t_num,t_seq,fit_intercept,
                         normalize,copy_X,fast_init,group_index,verbose)
    obj.initialize_paras(X,y)
    if family == 'binomial':
         y_new = obj.y
    else:
         y_new = y
         
    
    predict_error_cv = np.zeros((len(obj.t_seq),))
    for (fold,(train_index, test_index)) in enumerate(skf.split(X, y_new)):
         print('\x1b[6;30;42m' + 'Cross-validation: the %s-th fold'%(fold+1) + '\x1b[0m')
         X_train, X_test = X[train_index,:], X[test_index,:]
         y_train, y_test = y_new[train_index], y_new[test_index]
         obj.fit(X_train,y_train)
         
         if family == 'gaussian':
              predict_error_cv += predict_error_gaussian(X_test,y_test,obj.beta_,obj.intercept_)
              
         elif family == 'binomial':
              predict_error_cv += predict_error_binomial(X_test,y_test,obj.beta_,obj.intercept_)
              
    obj.predic_error_cv = predict_error_cv
    ## Pick up the regularization parameter which gives the minimum prediction error
    obj.ind_cv = np.argmin(predict_error_cv)
     
    ## Training on the whole dataset
    print('\x1b[0;30;46m' + 'Training on the whole dataset' + '\x1b[0m')
    obj.fit(X,y)
     
    return obj
     
              
              
              
         
         
    
        

    

     



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Jul  6 16:35:44 2018

@author: sxwxiaoxiao
"""

import numpy as np
from scipy import sparse
import warnings
import matplotlib.pyplot as plt
from sklearn import metrics
from sklearn.utils.validation import check_X_y
from sklearn.utils.sparsefuncs import mean_variance_axis, inplace_column_scale
FLOAT_DTYPES = (np.float64, np.float32, np.float16)

def sigmoid(x):
     
     """
     Sigmoid function.
     
     Parameters
     ----------
     x: vector or matrix
     
     Returns
     -------
     e^x / (1 + e^x)
     
     """
     return 1 / (1 + np.exp(-x))

def check_binomial(y):
     """
     Check if the y has two discrete values and transfer them to -1 and 1
     
     Parameters
     ----------
     y: Original response variable
     
     Returns
     -------
     The response variable with binomial values.
     """
     y_binomial = y.copy()
     y_unique = np.unique(y)
     if len(y_unique) != 2:
          raise ValueError('The response variable should be only have 2 discrete numbers!')
     else:
          y_binomial[np.where(y == y_unique[0])] = -1
          y_binomial[np.where(y == y_unique[1])] = 1
     return y_binomial
     

def safe_sparse_dot(a,b):
     """
     Dot product that handle the sparse matrix case correctly.
     
     Parameters
     ----------
     a : array or sparse matrix
     b : array or sparse matrix
     
     Returns
     -------
     dot_product : array or sparse matrix
        sparse if ``a`` or ``b`` is sparse.
     """
     if sparse.issparse(a) or sparse.issparse(b):
          return a * b
     else:
          return np.dot(a,b)
     

def _rescale_data(X, y, sample_weight):
     
    """
    Rescale data in terms of sample_weight
    
    Parameters
    ----------
    X: Design matrix
    y: Response variable
    
    Returns
    rescaled X and y
    """
    
    n_samples = X.shape[0]
    sample_weight = np.sqrt(sample_weight)
    sw_matrix = sparse.dia_matrix((sample_weight, 0),
                                  shape=(n_samples, n_samples))
    X = safe_sparse_dot(sw_matrix,X)
    y = safe_sparse_dot(sw_matrix,y)
    return X, y

def _rescale_data_binomial(a,sample_weight):
     
    """
    Rescale data in terms of sample_weight in logit model
    
    Parameters
    ----------
    a: vector or matrix
    
    Returns
    rescaled a
    """
    
    n_samples = a.shape[0]
    if sample_weight is not None:
         sw_matrix = sparse.dia_matrix((sample_weight, 0),
                                  shape=(n_samples, n_samples))
         a = safe_sparse_dot(sw_matrix,a)
    
    return a
     


def _preprocess_data(X, normalize=False, copy=True):
    
    """
    Preprocessing X if the normalize is True
    
    Parameters
    ----------
    X: Design matrix
    
    Returns
    normalized X with the variance of each column equal to 1 if normalize is True.
    """

    if normalize:
        if sparse.issparse(X):
            _, X_var = mean_variance_axis(X, axis=0)
            X_scale = np.sqrt(X_var)
            inplace_column_scale(X, 1. / X_scale) 
        else:   
            X_std = np.var(X,axis=0)
            XX = X.T / X_std[:,np.newaxis]
            X = XX.T
            
    return X


def linear_mle_tmle(X,y,group_index,sample_weight=None):
     
     """
     MLE estimator of linear model.
    
     Parameters
     ----------
     X: Design matrix
     y: Response variable
     
     Returns
     Intercept (=0 if fit_intercept is False),coefficients of MLE;
     the regularization parameter before which the variable is not selected
     """
     
     # Compute intercept
     intercept = np.mean(y)
     # Compute gradient
     d_beta = safe_sparse_dot(X.T,intercept-y) / X.shape[0]
     # Compute tmle
     G = len(np.unique(group_index))
     tmle = 1 / np.max([np.linalg.norm(d_beta[np.where(group_index==g)[0]],2) for g in range(0,G)])
     
     return intercept,d_beta,tmle



def logistic_mle_tmle(X,y,group_index,sample_weight):
     
     """
     MLE estimator of logit model.
    
     Parameters
     ----------
     X: Design matrix
     y: Binomial response variable
     
     Returns
     Intercept (=0 if fit_intercept is False),coefficients of MLE;
     the regularization parameter before which the variable is not selected
     """
     
     # Compute intercept
     y_pos = np.sum(sample_weight[y==1])
     y_neg = np.sum(sample_weight[y==-1])
     intercept = np.log(y_pos / y_neg)
     # Compute gradient
     tmp = -y / (1 + np.exp(intercept * y))
     tmp = _rescale_data_binomial(tmp,sample_weight)
     d_beta = safe_sparse_dot(X.T,tmp) / X.shape[0]
     # Compute tmle
     G = len(np.unique(group_index))
     tmle = 1 / np.max([np.linalg.norm(d_beta[np.where(group_index==g)[0]],2) for g in range(0,G)]) 
     
     return intercept,d_beta,tmle



def compute_alpha(X,kappa):
     """
     The step size of iteration
    
     Parameters
     ----------
     X: Design matrix
     kappa: Damping factor
     
     Returns
     step size
     """
     return X.shape[0] / kappa / np.linalg.norm(X,2) ** 2 

def gradient_linear_intercept(X,y,beta,intercept):
     """
     Gradient of the intercept of linear model
    
     Parameters
     ----------
     X: Design matrix
     y: Response variable
     beta: the coefficients at the current step 
     
     Returns
     the gradient of the intercept
     """
     return intercept + np.mean(safe_sparse_dot(X,beta) - y)

def gradient_logistic_intercept(X,y,beta,intercept,sample_weight):
     """
     Gradient of the intercept of logit model
    
     Parameters
     ----------
     X: Design matrix
     y: Response variable
     beta: the coefficients at the current step 
     
     Returns
     the gradient of the intercept
     """
     Xbeta_intercept = safe_sparse_dot(X,beta) + intercept
     tmp = - y / (1 + np.exp(Xbeta_intercept * y))
     tmp = _rescale_data_binomial(tmp,sample_weight)
     return np.mean(tmp)

def gradient_linear(X,y,beta,intercept):
     """
     Gradient of the intercept of linear model
    
     Parameters
     ----------
     X: Design matrix
     y: Response variable
     beta: the coefficients at the current step 
     intercept: the intercept at the current step
     
     Returns
     the gradient of the beta
     """
     
     Xbeta = safe_sparse_dot(X,beta)
     return safe_sparse_dot(X.T,Xbeta + intercept - y) / X.shape[0]



def gradient_logistic(X,y,beta,intercept,sample_weight):
     """
     Gradient of the intercept of linear model
    
     Parameters
     ----------
     X: Design matrix
     y: Response variable
     beta: the coefficients at the current step 
     intercept: the intercept at the current step
     
     Returns
     the gradient of the beta
     """
     Xbeta_intercept = safe_sparse_dot(X,beta) + intercept
     tmp = - y / (1 + np.exp(Xbeta_intercept * y))
     tmp = _rescale_data_binomial(tmp,sample_weight)
     return np.sum(tmp) / np.sum(sample_weight),safe_sparse_dot(X.T,tmp) / X.shape[0] 


def shrinkage(z):
     """
     Soft-thresholding
    
     Parameters
     ----------
     z: rho + beta / kappa ( rho \in \partial(||z||_1) )
     
     Returns
     beta = argmin_x 1/2 * || x - z ||^2 + ||z||_1
     """
     return np.sign(z) * np.array([np.max([0,np.abs(zz)-1]) for zz in z])

def shrinkage_group(z,group_index):
     """
     Group wise Soft-thresholding
    
     Parameters
     ----------
     z_g (g = 1,..,G): rho_g + beta_g / kappa ( rho_g \in \partial(||z_g||_2) )
     
     Returns
     beta_g = argmin_x 1/2 * || x - z_g ||^2 + ||z_g||_2
     """
     z_shrink = np.zeros((len(z),))
     G = len(np.unique(group_index))
     for g in range(G):
          g_ind = np.where(group_index == g)[0]
          z_gind_norm = np.linalg.norm(z[g_ind],2)
          if z_gind_norm == 0:
               z_shrink[g_ind] = 0
          else:
               z_shrink[g_ind] = np.max([0, 1 - 1 / z_gind_norm]) * z[g_ind]
          
     return z_shrink
          


def variable_path(beta_current,var_hist,var_step,num_iter,group_index):
     """
     record the variables that are selected and the step at which they are selected
    
     Parameters
     ----------
     beta_current: the coefficients at the current step
     var_hist: the history set of variables that have been selected at least once
     var_step: the earlest step at which each variable in var_hist is selected.
     
     Returns
     updated var_hist and var_step
     """
     var_current = np.unique(group_index[np.where(np.abs(beta_current) > 1e-6)])
     var_new = np.setdiff1d(var_current,var_hist)
     if len(var_new) > 0:
          var_hist.extend(list(var_new))
          var_step.extend(list(num_iter * var_new))
     return var_hist,var_step
     
def predict_error_gaussian(X,y,beta_,intercept_):
     
     """
     Compute the prediction error of the path in linear model.
     
     Parameters
     ----------
     X: the design matrix, with shape (n_samples,n_features)
     y: true response variable, with shape (n_samples,)
     beta_: the path coefficients, with shape (n_features, n_knots)
     intercept_: the path intercept, with shape(n_knots,)
     
     Returns
     prediction error of the path
     """
     n_samples,n_features = X.shape
     if y.shape[0] != n_samples:
          raise ValueError('The row number design matrix and the label should be the same!')
     if beta_.shape[0] != n_features:
          raise ValueError('The row number of beta_ should be equal to the p (the number of features)!')
     if intercept_.shape[0] != beta_.shape[1]:
          raise ValueError('The number of knots in beta_ and intercept_ should be the same!')
     
     # estimate y     
     y_est_path = X.dot(beta_) + intercept_
     # y_est_path - y
     err_path = (y_est_path.T - y.T)
     
     return np.linalg.norm(err_path,ord=2,axis=1)

def predict_error_binomial(X,y,beta_,intercept_):
     
     """
     Compute the prediction error of the path in logit model.
     
     Parameters
     ----------
     X: the design matrix, with shape (n_samples,n_features)
     y: true response variable, with shape (n_samples,)
     beta_: the path coefficients, with shape (n_features, n_knots)
     intercept_: the path intercept, with shape(n_knots,)
     
     Returns
     prediction error of the path
     """
     n_samples,n_features = X.shape
     if y.shape[0] != n_samples:
          raise ValueError('The row number design matrix and the label should be the same!')
     if beta_.shape[0] != n_features:
          raise ValueError('The row number of beta_ should be equal to the p (the number of features)!')
     
     # compute probability
     xbeta_path = X.dot(beta_) + intercept_
     prob_path = sigmoid(xbeta_path)
     # estimat y
     y_est_path = (prob_path >= 0.5) * 1
     y_est_path[y_est_path == 0] = -1
     # y_est_path - y
     err_path = (y_est_path.T - y.T)
     
     if err_path.ndim == 1:
          err_path = err_path[np.newaxis,:]
     
     return np.sum(err_path!=0,axis=1)


class LBI():
     
     """
      Base class for Linearized Bregman Iteration.
     """

    
     def __init__(self,kappa=10,alpha=None,t0=None,t_ratio=100,t_num=500,t_seq=None,
                fit_intercept=False,normalize=False,copy_X=True,fast_init=False,
                group_index=None,verbose=True):
          
          """
           Initialization of the parameters.
          """
          try:
               self.kappa = float(kappa)
          except:
               raise ValueError('The dtype of kappa should be convertible to float')
          if self.kappa <= 0:
               raise ValueError('The Damping Factor kappa should be geater than 0!')
             
          self.alpha = alpha
          self.t0 = t0
          try:
               self.t_ratio = int(t_ratio)
               self.t_num = int(t_num)
          except:
               raise ValueError('The dtype of t_ratio and t_num should be convertible to int')
          if self.t_ratio <= 1 or self.t_num < 1:
               raise ValueError('The t_ratio (t_num) should be geater (or equal) than 1!')
          
          self.t_seq = t_seq
          
          try:
               self.fit_intercept = bool(fit_intercept)
               self.normalize = bool(normalize)
               self.copy_X = bool(copy_X)
               self.fast_init = bool(fast_init)
               self.verbose = bool(verbose)
          except:
               raise ValueError('The dtype of fit_intercept,normalize,copy_X,fast_init,auc and verbose should be convertible to bool!')
          self.group_index = group_index
          
          
     def initialize_paras(self,X,y,sample_weight):
          
          """
           Initialization of the parameters which don't provided, 
           i.e. step size, t_seq and group_index.
          """
          
          # Check the format of X and y
          X, y = check_X_y(X, y, accept_sparse=['csr', 'csc', 'coo'],
                           y_numeric=True, multi_output=True)
        
          # The sample_weight should be a vector
          if sample_weight is not None:
              try:
                   self.sample_weight = np.abs(np.array([float(sw) for sw in sample_weight]))
              except:
                   raise ValueError('The sample weight should be a 1D array and can be convertible to float!')
              if len(sample_weight) != X.shape[0]:
                   raise ValueError("The dimension of sample weight should be equal to the number of samples!")
              if np.max(sample_weight) == 0:
                   raise ValueError('At least one sample has the weight greater than 0!')
              else:
                   if np.isinf(np.max(sample_weight)):
                        inf_ind = np.isinf(sample_weight)
                        self.sample_weight = np.zeros((X.shape[0],))
                        self.sample_weight[inf_ind] = 1
                   else:
                        self.sample_weight = sample_weight / np.mean(sample_weight)
          else:
               self.sample_weight = np.ones((X.shape[0],))
               
          
          
          if self.t0 is not None:
               try:
                    self.t0 = float(self.t0)
               except:
                    raise ValueError('The dtype of t0 should be convertible to float!')
               if self.t0 < 0:
                    raise ValueError('The t0 should be geater than 0!')
          
          if self.t_seq is not None:
               try:
                    self.t_seq = np.array([float(t) for t in self.t_seq])
               except:
                    raise ValueError('The dtype of regularization parameter should be convertible to float!')
               if np.min(self.t_seq) < 0 or np.isinf(np.max(self.t_seq)):
                    raise ValueError('The vector of regularization parameters should be greater than 0 and less than inf!')
        
          if self.group_index is None:
             self.group_index = np.array(range(0,X.shape[1]))
          else:
             try:
                  self.group_index = np.asarray([float(gi) for gi in self.group_index])
             except:
                  raise ValueError('The group index should be a vector of number that the order of each one indicating the which group the corresponding variable belongs to!')
             if len(self.group_index) != X.shape[1]:
                  raise ValueError('The dimension of group_index must be equal to p (the number of features!')
             else:
                  index_sort = np.unique(np.sort(self.group_index))
                  self.group_index = np.array([np.where(index_sort==ind)[0][0] for ind in self.group_index])
         
          group_index_unique = np.unique(self.group_index)
          if len(group_index_unique) < len(self.group_index):
               self.group_ornot = True
          else:
               self.group_ornot = False
               
          
          
          
             
          
     def initialize_estimator(self,X,y):
          
          """
           The initialization of the beta,z in the regularization solution path.
           
          """
          
          (n_samples,n_features) = X.shape
          n_samples_y = len(y)
          if n_samples_y != n_samples:
               raise ValueError('The number of samples of X and y should be the same!')
          
          # Initialize the path 
          self.beta_ = np.zeros((n_features,self.t_num))
          self.intercept_ = np.zeros((self.t_num,))
          self.z_ = np.zeros((n_features,self.t_num))
          self.var_hist = []
          self.var_order = []
          
          
     def MLE_initialization(self,X,y,sample_weight):
          
          """
           The initialization of the beta,z in the regularization solution path.
           
          """
          mle_dict = {'gaussian':linear_mle_tmle,'binomial':logistic_mle_tmle}
          self.beta = np.zeros((X.shape[1],))
          self.z = np.zeros((X.shape[1],))
          self.intercept = 0
          
          if self.fast_init is True:
               intercept,d_beta,t_mle = mle_dict[self.family](X,y,self.group_index,sample_weight)
               self.t_mle = t_mle
               if self.t0 >= self.t_mle:
                    self.intercept = intercept
                    self.z = -self.t_mle * d_beta
                    self.t0 = t_mle
               else:
                    self.t0 = 0
                    warnings.warn('Not use MLE to initialize since it causes the t_seq to be incomplete!')  
          else:
               self.t0 = 0
               
               
     
     
     def ROC_AUC(self,ind_true,roc_plot=True):
          
          """ 
          Compute the AUC of the regularization solution path
          The ROC Curve is also plotted if roc_plot is True.
          
          """
          if self.group_ornot:
               n_features = len(np.unique(self.group_index))
          else:
               n_features = len(self.beta)
          try:
               ind_true = np.unique(np.asarray([int(it) for it in ind_true]))
               ind_true = np.reshape(ind_true,(ind_true.shape[0],))
          except:
               raise ValueError('The ind_true is the index of the true signal set, which should be int!')
          if len(ind_true) <= n_features:
               if np.max(ind_true) >= n_features or np.min(ind_true) < 0:
                    raise ValueError('The index should be >=0 and <= p (the number of features)!')
          else:
               raise ValueError('The number of true signal set should be less or equal than p!')
               
          # True label
          labels = np.zeros((n_features,1))
          labels[ind_true] = 1
          # Score which measures the order of selected variables
          scores = np.max(np.min(self.var_order) / np.max(self.var_order) - 0.01,0) * np.ones((n_features,))
          scores[self.var_hist] = np.sort(self.var_order)[::-1] / np.max(self.var_order)
          # Compute AUC
          fpr, tpr, thresholds = metrics.roc_curve(labels, scores, pos_label=1)
          AUC = metrics.auc(fpr, tpr)
    
          # Plot the ROC Curve
          if roc_plot is True:
               plt.figure()
               lw = 2
               plt.plot(fpr, tpr, color='red',
                        lw=lw, label='ROC curve (area = %0.5f)' % AUC)
               plt.plot([0, 1], [0, 1], color='black', lw=lw, linestyle='--')
               plt.xlim([-0.01, 1.0])
               plt.ylim([-0.01, 1.05])
               plt.xlabel('False Positive Rate')
               plt.ylabel('True Positive Rate')
               plt.title('Receiver Operating Characteristic')
               plt.legend(loc="lower right")
               plt.grid(True)
               plt.show()
          
          return AUC 
     
     def Solution_path(self,ind_true=None):
          """ 
          Plot the solution path
          
          """
          if ind_true is None:
               ind_true = []
          else:
               if self.group_ornot is True:
                    ind_true = [list(np.where(self.group_index == ind)[0]) for ind in ind_true]
                    ind_true = sum(ind_true,[])
          plt.figure()
          plt.rc('text', usetex=True)
          for i in range(self.beta_.shape[0]):
               if i in ind_true:
                    plt.plot(self.t_seq,self.beta_[i,:],'-')
               else:
                    plt.plot(self.t_seq,self.beta_[i,:],'--')
                    
          plt.xlim([-0.01, np.max(self.t_seq) * 1.01])
          plt.ylim([np.min(self.beta_) - 0.05 * np.abs(np.min(self.beta_)), np.max(self.beta_) + 0.05 * np.abs(np.max(self.beta_))])
          plt.xlabel('regularization parameter t')
          plt.ylabel(r'$\beta$')
          plt.grid(True)
          plt.title('Regularization Solution Path')
          plt.show()
          
     
          
     
     
     


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Jul 20 12:24:25 2018

@author: sxwxiaoxiao
"""

from setuptools import setup  # Always prefer setuptools over distutils
from codecs import open  # To use a consistent encoding
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the relevant file
with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='libra_py_001_04',
    version='0.0.1',
    description='Split Linearized Bregman Iteration',
    long_description=long_description,
    url='https://github.com/tansey/smoothfdr',
    author='Xinwei Sun',
    author_email='sxwxiaoxiaohehe@pku.edu.cn',
    license='MIT',

    classifiers=[
        'Development Status :: 1 - Planning',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering',
        'License :: Free For Educational Use',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
    ],
    keywords='sparsity regularization path Lasso variable-selection',

    packages=['libra_py_001_04'],
    package_dir={'libra_py_001_04': 'libra_py_001_04'},
    install_requires=[
              'numpy',
              'scipy', 
              'matplotlib', 
              'scikit-learn'],
   
)
